@@ -14,4 +14,4 @@
 """ Provides an abstraction for files and folders in a file system.
     Part of the AppTools project of the Enthought Tool Suite.
 """
-from api import *
+from apptools.io.api import *
