@@ -2,28 +2,44 @@
 
 """
 # Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2005, Enthought, Inc.
+# Copyright (c) 2005-2015, Enthought, Inc.
 # License: BSD Style.
 
+import base64
+import pickle
 import unittest
 import math
+import os
+import tempfile
 
 import numpy
 
 from traits.api import Bool, Int, Long, Array, Float, Complex, Any, \
-     Str, Unicode, Instance, Tuple, List, Dict, HasTraits
-from tvtk.api import tvtk
+    Str, Unicode, Instance, Tuple, List, Dict, HasTraits
+
+try:
+    from tvtk.api import tvtk
+except ImportError:
+    TVTK_AVAILABLE = False
+else:
+    TVTK_AVAILABLE = True
 
 from apptools.persistence import state_pickler
 
 
 # A simple class to test instances.
-class A:
+class A(object):
+
     def __init__(self):
         self.a = 'a'
 
+# NOTE: I think that TVTK specific testing should be moved to the
+#       TVTK package.
+
+
 # A classic class for testing the pickler.
 class TestClassic:
+
     def __init__(self):
         self.b = False
         self.i = 7
@@ -34,20 +50,21 @@ class TestClassic:
         self.s = 'String'
         self.u = u'Unicode'
         self.inst = A()
-        self.tuple = (1,2,'a', A())
+        self.tuple = (1, 2, 'a', A())
         self.list = [1, 1.1, 'a', 1j, self.inst]
-        self.pure_list = range(5)
+        self.pure_list = list(range(5))
         self.dict = {'a': 1, 'b': 2, 'ref': self.inst}
-        self.numeric = numpy.ones((2,2,2), 'f')
+        self.numeric = numpy.ones((2, 2, 2), 'f')
         self.ref = self.numeric
-        self._tvtk = tvtk.Property()
+        if TVTK_AVAILABLE:
+            self._tvtk = tvtk.Property()
 
 
 # A class with traits for testing the pickler.
 class TestTraits(HasTraits):
     b = Bool(False)
     i = Int(7)
-    l = Long(1234567890123456789L)
+    l = Long(12345678901234567890)
     f = Float(math.pi)
     c = Complex(complex(1.01234, 2.3))
     n = Any
@@ -56,14 +73,16 @@ class TestTraits(HasTraits):
     inst = Instance(A)
     tuple = Tuple
     list = List
-    pure_list = List(range(5))
+    pure_list = List(list(range(5)))
     dict = Dict
-    numeric = Array(value=numpy.ones((2,2,2), 'f'))
+    numeric = Array(value=numpy.ones((2, 2, 2), 'f'))
     ref = Array
-    _tvtk = Instance(tvtk.Property, ())
+    if TVTK_AVAILABLE:
+        _tvtk = Instance(tvtk.Property, ())
+
     def __init__(self):
         self.inst = A()
-        self.tuple = (1,2,'a', A())
+        self.tuple = (1, 2, 'a', A())
         self.list = [1, 1.1, 'a', 1j, self.inst]
         self.dict = {'a': 1, 'b': 2, 'ref': self.inst}
         self.ref = self.numeric
@@ -81,8 +100,26 @@ class TestDictPickler(unittest.TestCase):
         obj.list[0] = 2
         obj.tuple[-1].a = 't'
         obj.dict['a'] = 10
-        obj._tvtk.set(point_size=3, specular_color=(1, 0, 0),
-                      representation='w')
+        if TVTK_AVAILABLE:
+            obj._tvtk.trait_set(
+                point_size=3, specular_color=(1, 0, 0),
+                representation='w'
+            )
+
+    def _check_instance_and_references(self, obj, data):
+        """Asserts that there is one instance and two references in the state.
+        We need this as there isn't a guarantee as to which will be the
+        reference and which will be the instance.
+        """
+        inst = data['inst']
+        list_end = data['list']['data'][-1]
+        dict_ref = data['dict']['data']['ref']
+        all_inst = [inst, list_end, dict_ref]
+        types = [x['type'] for x in all_inst]
+        self.assertEqual(types.count('instance'), 1)
+        self.assertEqual(types.count('reference'), 2)
+        inst_state = all_inst[types.index('instance')]
+        self.assertEqual(inst_state['data']['data']['a'], 'b')
 
     def verify(self, obj, state):
         data = state['data']
@@ -96,33 +133,33 @@ class TestDictPickler(unittest.TestCase):
         self.assertEqual(data['n'], obj.n)
         self.assertEqual(data['s'], obj.s)
         self.assertEqual(data['u'], obj.u)
-        if isinstance(obj, HasTraits):
-            self.assertEqual(data['inst']['type'], 'reference')
-        else:
-            self.assertEqual(data['inst']['type'], 'reference')
         tup = data['tuple']['data']
         self.assertEqual(tup[:-1], obj.tuple[:-1])
         self.assertEqual(tup[-1]['data']['data']['a'], 't')
         lst = data['list']['data']
         self.assertEqual(lst[:-1], obj.list[:-1])
 
-        if isinstance(obj, HasTraits):
-            self.assertEqual(lst[-1]['type'], 'instance')
-        else:
-            self.assertEqual(lst[-1]['data']['data']['a'], 'b')
-
         pure_lst = data['pure_list']['data']
         self.assertEqual(pure_lst, obj.pure_list)
         dct = data['dict']['data']
         self.assertEqual(dct['a'], obj.dict['a'])
         self.assertEqual(dct['b'], obj.dict['b'])
-        self.assertEqual(dct['ref']['type'], 'reference')
 
-        junk = state_pickler.gunzip_string(data['numeric']['data'].decode('base64'))
-        num = numpy.loads(junk)
+        self._check_instance_and_references(obj, data)
+
+        num_attr = 'numeric' if data['numeric']['type'] == 'numeric' else 'ref'
+        decodestring = getattr(base64, 'decodebytes', base64.decodestring)
+        junk = state_pickler.gunzip_string(
+            decodestring(data[num_attr]['data'])
+        )
+        num = pickle.loads(junk)
         self.assertEqual(numpy.alltrue(numpy.ravel(num == obj.numeric)), 1)
 
-        self.assertEqual(data['ref']['type'], 'reference')
+        self.assertTrue(data['ref']['type'] in ['reference', 'numeric'])
+        if data['ref']['type'] == 'numeric':
+            self.assertEqual(data['numeric']['type'], 'reference')
+        else:
+            self.assertEqual(data['numeric']['type'], 'numeric')
         self.assertEqual(data['ref']['id'], data['numeric']['id'])
 
     def verify_unpickled(self, obj, state):
@@ -164,10 +201,11 @@ class TestDictPickler(unittest.TestCase):
         self.assertEqual(numpy.alltrue(numpy.ravel(num == obj.numeric)), 1)
         self.assertEqual(id(state.ref), id(num))
 
-        _tvtk = state._tvtk
-        self.assertEqual(_tvtk.representation, obj._tvtk.representation)
-        self.assertEqual(_tvtk.specular_color, obj._tvtk.specular_color)
-        self.assertEqual(_tvtk.point_size, obj._tvtk.point_size)
+        if TVTK_AVAILABLE:
+            _tvtk = state._tvtk
+            self.assertEqual(_tvtk.representation, obj._tvtk.representation)
+            self.assertEqual(_tvtk.specular_color, obj._tvtk.specular_color)
+            self.assertEqual(_tvtk.point_size, obj._tvtk.point_size)
 
     def verify_state(self, state1, state):
         self.assertEqual(state.__metadata__,
@@ -180,27 +218,36 @@ class TestDictPickler(unittest.TestCase):
         self.assertEqual(state.n, state1.n)
         self.assertEqual(state.s, state1.s)
         self.assertEqual(state.u, state1.u)
+        # The ID's need not be identical so we equate them here so the
+        # tests pass.  Note that the ID's only need be consistent not
+        # identical!
+        if TVTK_AVAILABLE:
+            instances = ('inst', '_tvtk')
+        else:
+            instances = ('inst',)
+
+        for attr in instances:
+            getattr(state1, attr).__metadata__['id'] = \
+                getattr(state, attr).__metadata__['id']
+
+        if TVTK_AVAILABLE:
+            self.assertEqual(state1._tvtk, state._tvtk)
+
+        state1.tuple[-1].__metadata__['id'] = \
+            state.tuple[-1].__metadata__['id']
         self.assertEqual(state.inst.__metadata__, state1.inst.__metadata__)
 
         self.assertEqual(state.tuple, state1.tuple)
-        lst = state.list
         self.assertEqual(state.list, state1.list)
 
         self.assertEqual(state.pure_list, state1.pure_list)
 
-        dct = state.dict
         self.assertEqual(state.dict, state1.dict)
 
-        num = state.numeric
-        self.assertEqual((state1.numeric ==state.numeric).all(), True)
+        self.assertEqual((state1.numeric == state.numeric).all(), True)
         self.assertEqual(id(state.ref), id(state.numeric))
         self.assertEqual(id(state1.ref), id(state1.numeric))
 
-        # The ID's need not be identical so we equate them here so the
-        # tests pass.  Note that the ID's only need be consistent not
-        # identical!
-        state1._tvtk.__metadata__['id'] = state._tvtk.__metadata__['id']
-        self.assertEqual(state1._tvtk, state._tvtk)
 
     def test_has_instance(self):
         """Test to check has_instance correctness."""
@@ -211,13 +258,14 @@ class TestDictPickler(unittest.TestCase):
         r = state_pickler.get_state(l)
         self.assertEqual(r.has_instance, True)
         self.assertEqual(r[1].__metadata__['has_instance'], True)
-        d = {'a': l, 'b':1}
+        d = {'a': l, 'b': 1}
         r = state_pickler.get_state(d)
         self.assertEqual(r.has_instance, True)
         self.assertEqual(r['a'].has_instance, True)
         self.assertEqual(r['a'][1].__metadata__['has_instance'], True)
 
         class B:
+
             def __init__(self):
                 self.a = [1, A()]
 
@@ -227,7 +275,6 @@ class TestDictPickler(unittest.TestCase):
         self.assertEqual(r.a.has_instance, True)
         self.assertEqual(r.a[1].__metadata__['has_instance'], True)
 
-
     def test_pickle_classic(self):
         """Test if classic classes can be pickled."""
         t = TestClassic()
@@ -236,8 +283,7 @@ class TestDictPickler(unittest.TestCase):
         state = state_pickler.StatePickler().dump_state(t)
 
         # First check if all the attributes are handled.
-        keys = state['data']['data'].keys()
-        keys.sort()
+        keys = sorted(state['data']['data'].keys())
         expect = [x for x in t.__dict__.keys() if '__' not in x]
         expect.sort()
         self.assertEqual(keys, expect)
@@ -311,8 +357,7 @@ class TestDictPickler(unittest.TestCase):
         state = state_pickler.StatePickler().dump_state(t)
 
         # First check if all the attributes are handled.
-        keys = state['data']['data'].keys()
-        keys.sort()
+        keys = sorted(state['data']['data'].keys())
         expect = [x for x in t.__dict__.keys() if '__' not in x]
         expect.sort()
         self.assertEqual(keys, expect)
@@ -345,8 +390,11 @@ class TestDictPickler(unittest.TestCase):
 
     def test_reference_cycle(self):
         """Test if reference cycles are handled when setting the state."""
-        class A: pass
-        class B: pass
+        class A:
+            pass
+
+        class B:
+            pass
         a = A()
         b = B()
         a.a = b
@@ -357,6 +405,15 @@ class TestDictPickler(unittest.TestCase):
         z.a.b = z
         state_pickler.set_state(z, state)
 
+    def test_get_state_on_tuple_with_numeric_references(self):
+        num = numpy.zeros(10, float)
+        data = (num, num)
+        # If this just completes without error, we are good.
+        state = state_pickler.get_state(data)
+        # The two should be the same object.
+        self.assertTrue(state[0] is state[1])
+        numpy.testing.assert_allclose(state[0], num)
+
     def test_state_is_saveable(self):
         """Test if the state can be saved like the object itself."""
         t = TestClassic()
@@ -377,12 +434,15 @@ class TestDictPickler(unittest.TestCase):
     def test_get_pure_state(self):
         """Test if get_pure_state is called first."""
         class B:
+
             def __init__(self):
                 self.a = 'dict'
+
             def __get_pure_state__(self):
-                return {'a':'get_pure_state'}
+                return {'a': 'get_pure_state'}
+
             def __getstate__(self):
-                return {'a':'getstate'}
+                return {'a': 'getstate'}
         b = B()
         s = state_pickler.get_state(b)
         self.assertEqual(s.a, 'get_pure_state')
@@ -393,6 +453,17 @@ class TestDictPickler(unittest.TestCase):
         s = state_pickler.get_state(b)
         self.assertEqual(s.a, 'dict')
 
+    def test_dump_to_file_str(self):
+        """Test if dump can take a str as file"""
+        obj = A()
+
+        filepath = os.path.join(tempfile.gettempdir(), "tmp.file")
+
+        try:
+            state_pickler.dump(obj, filepath)
+        finally:
+            os.remove(filepath)
+
 
 if __name__ == "__main__":
     unittest.main()
